
// Floor.java
// Andrew Davison, June 2007, ad@fivedots.coe.psu.ac.th

/* Draw a green and blue checkboard floor with a red square 
   at its center and numbers along its z- and z- axes.

   This code is almost a direct translation of the CheckerFloor class
   in the Java 3D Checkers3D example in chapter 15 of KGPJ; 
   online at http://fivedots.coe.psu.ac.th/~ad/jg/ch8/).
*/

import javax.media.opengl.*;
import javax.media.opengl.glu.*;
import com.sun.opengl.util.*;


public class Floor
{
  private final static int FLOOR_LEN = 20;  // should be even
  private final static int BLUE_TILE = 0;   // floor tile colour types
  private final static int GREEN_TILE = 1;


  // OpenGL
  private GL gl;
  private GLUT glut;


  public Floor(GL gl, GLUT glut)
  { 
    this.gl = gl;
    this.glut = glut;
  } // end of Floor()


  public void draw()
  /* Create tiles, the origin marker, then the axes labels.
     The tiles are in a checkboard pattern, alternating between
     green and blue.
  */
  {
    gl.glDisable(GL.GL_LIGHTING);

    drawTiles(BLUE_TILE);   // blue tiles
    drawTiles(GREEN_TILE);  // green
    addOriginMarker();
    labelAxes();

    gl.glEnable(GL.GL_LIGHTING);
  }  // end of draw()


  private void drawTiles(int drawType)
  /* Create a series of quads, all with the same colour. They are
     spaced out over a FLOOR_LEN*FLOOR_LEN area, with the area centered
     at (0,0) on the XZ plane, and y==0.
  */
  {
    if (drawType == BLUE_TILE)
      gl.glColor3f(0.0f, 0.1f, 0.4f);
    else  // green
      gl.glColor3f(0.0f, 0.5f, 0.1f);

    gl.glBegin(GL.GL_QUADS);
    boolean aBlueTile;
    for(int z=-FLOOR_LEN/2; z <= (FLOOR_LEN/2)-1; z++) {
      aBlueTile = (z%2 == 0)? true : false;    // set colour type for new row
      for(int x=-FLOOR_LEN/2; x <= (FLOOR_LEN/2)-1; x++) {
        if (aBlueTile && (drawType == BLUE_TILE))  // blue tile and drawing blue
          drawTile(x, z);
        else if (!aBlueTile && (drawType == GREEN_TILE))   // green
          drawTile(x, z);
        aBlueTile = !aBlueTile;
      }
    }
    gl.glEnd();
  }  // end of drawTiles()


  private void drawTile(int x, int z)
  /* Coords for a single blue or green square; 
    its top left hand corner at (x,0,z). */
  {
    // points created in counter-clockwise order
    gl.glVertex3f(x, 0.0f, z+1.0f);   // bottom left point
    gl.glVertex3f(x+1.0f, 0.0f, z+1.0f);
    gl.glVertex3f(x+1.0f, 0.0f, z);
    gl.glVertex3f(x, 0.0f, z);
  }  // end of drawTile()


  private void addOriginMarker()
  /* A red square centered at (0,0.01,0), of length 0.5, lieing
     flat on the XZ plane. */
  {  
    gl.glColor3f(0.8f, 0.4f, 0.3f);   // medium red
    gl.glBegin(GL.GL_QUADS);

    // points created counter-clockwise, a bit above the floor
    gl.glVertex3f(-0.25f, 0.01f, 0.25f);  // bottom left point
    gl.glVertex3f(0.25f, 0.01f, 0.25f);
    gl.glVertex3f(0.25f, 0.01f, -0.25f);    
    gl.glVertex3f(-0.25f, 0.01f, -0.25f);

    gl.glEnd();
  } // end of addOriginMarker();


  private void labelAxes()
  /* Place numbers along the X- and Z-axes at the integer positions.
     The axes are drawn using a stroke font, so the shape can be scaled,
     and the thickness of the line increased (to make it look 'bold').
  */
  {
    gl.glColor3f(1.0f, 1.0f, 1.0f);   // white
    gl.glLineWidth(3.0f);  // thicken the line

    for (int i=-FLOOR_LEN/2; i <= FLOOR_LEN/2; i++)
      drawAxisText(""+i, (float)i, 0.0f, 0.0f);  // along x-axis

    for (int i=-FLOOR_LEN/2; i <= FLOOR_LEN/2; i++)
      drawAxisText(""+i, 0.0f, 0.0f, (float)i);  // along z-axis

    gl.glLineWidth(1.0f);  // reset line width
  }  // end of labelAxes()



  private void drawAxisText(String txt, float x, float y, float z) 
  /* Draw txt at (x,y,z), with the text centered in the x-direction,
     facing along the +z axis.
  */
  {
    gl.glPushMatrix();
    gl.glTranslatef(x, y, z);    // position the text
    gl.glScalef(0.0015f, 0.0015f, 0.0015f);    // reduce rendering size

    // center text on the x-axis
    float width = glut.glutStrokeLength(GLUT.STROKE_MONO_ROMAN, txt);
    gl.glTranslatef(-width/2.0f, 0, 0);

    // render the text using a stroke font
    for (int i = 0; i < txt.length(); i++) {
      char ch = txt.charAt(i);
      glut.glutStrokeCharacter(GLUT.STROKE_MONO_ROMAN, ch);
    }

    gl.glPopMatrix();   // restore model view
  } // end of drawAxisText()


} // end of Floor class

